/* 
 * Author: Yueping Zhang, Internet Research Lab (IRL), Texas A&M University
 * Email: yueping@cs.tamu.edu
 * File Name: jetmax-queue.h
 * Functionality: header file of the JetMax router
 * Last modified on Nov. 15, 2005
 */

#include "jetmax-source.h"
#include "delay.h"

#define	MIN_DISTANCE 0.01
#define SIZE 100000 

class JetMaxQueue;

/* 
 * JetMax parameters
 */
struct JetMaxp {
	int		rid_c;			// current router id
  	double	rate_total;   	// combined rate of incoming traffic
	double	pkt_loss;		// packet loss calculated by only counting flows congested by the current router
  	double	congested;      // the total amount of data from responsive flows arrived during the current interval
  	double	unresponsive;	// the amount of arrived data from unresponsive flows
	double	N_congested;    // number of congested flows
	double	C;				// capacity of the out-going link
	double	g;              // ideal fair rate
  	int		pkt_loss_seqno;	// packet loss sequence number
};

/*
 * Class of the JetMax router
 */
class JetMaxQueue : public Queue { 
public:	
		JetMaxQueue();      // constructor

protected:
		void	enque(Packet* pkt);			// enqueue the current packet
		Packet*	deque();					// return the packet on top of the queue
		void	recv(Packet* p, Handler*);  // receive function
		void    do_on_packet_arrival(hdr_JetMax*);// called upon each packet arrival
		void	write_packet(hdr_JetMax*);  // overwrite packet header
		void	compute_ack(hdr_JetMax*);   // calculate the feedback information
		bool	less_than_min(hdr_JetMax*);       
		bool	is_loss_zero(hdr_JetMax*);

		PacketQueue*	q_; 				// underlying FIFO queue
		JetMaxp	JetMaxp_;                   // local variables
		
		double	t_start_;					// start time of the current interval
		double	t_finish_;					// the time when the queue stop enqueuing
  		double	duration_;                  // *actual* length of the last delta interval
		double	delta_;						// *designated* length of the last delta interval
		double  gamma_;                     // desired link utilization level

		int		is_first_pkt_;				// indicate whether the first packet has arrived
		int		approval_;                  // whether approve the proposed rate or not
		int		qID_;                       // router ID
		int		cur_interval_;              // current packet loss interval
};
