/* 
 * Author: Yueping Zhang, Internet Research Lab (IRL), Texas A&M University
 * Email: yueping@cs.tamu.edu
 * File Name: jetmax-sink.h
 * Functionality: header file of the JetMax receiver
 * Last modified on Nov. 15, 2005
 */

#include "jetmax-source.h"

#define QSIZE 100000
//#define RANDOM_DELAY 1

class JetMaxSinkAgent;

/* 
 * Timer used to schedule ACKs
 */
class JetMaxAckTimer : public TimerHandler {
public:
	JetMaxAckTimer(JetMaxSinkAgent *a) : TimerHandler() { 
		a_ = a; 
	}
	virtual void expire(Event *e);
protected:
	JetMaxSinkAgent *a_; 
};

/*
 * A local queue that all ACKs will pass through
 */
struct ackQ
{
	int				isEmpty;
	Packet*			pkt;
};

/*
 * The JetMax receiver agent
 */
class JetMaxSinkAgent : public Agent {
		friend class JetMaxAckTimer;
public:
		JetMaxSinkAgent();
		void	recv(Packet*, Handler*);  // called upon each packet arrival
		void	sendpkt(Packet *);        // send out the ACK
		int		command(int argc, const char*const* argv);

		JetMaxAckTimer	ack_timer;        // timer for scheduling the next ACK to be sent
		struct ackQ*	ackQueue;         // force all ACK to pass through this local queue to generate time-varying delay
		int			tail;                 // position of the last packet in the queue
		int			head;                 // position of the first packet in the queue
		void		AckEnque(Packet *);   // enque the ACK
		Packet*		AckDeque();           // deque the ACK
		double		next_ack_time;        // time to send the next ACK
		int			interval_num;         // number of undelayed ACKs between two delay spikes
		int			cur_num;              // number of undelayed ACKs in the current interval
		int			isFirstPkt;           // 1 -- the current packet is the first one; 0 -- otherwise
}; 
